package jp.co.sds_corp.tool.shell;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import com.jcoverage.coverage.Instrumentation;
import com.jcoverage.coverage.InstrumentationFactory;

public class ShellInstrumentBuilder
{
	String shell;

	public ShellInstrumentBuilder(String shell) {
		this.shell = shell;
	}

	public Instrumentation getInstrumentation() throws Exception {
		File file = new File(shell);
		if (!file.exists()) {
			return null;
		}
		InstrumentationFactory factory = InstrumentationFactory.getInstance();
		Instrumentation inst = factory.newInstrumentation(shell);

		Set<Integer> lineNumbers = parseLineNumbers(shell);
//		inst.setSourceLineNumbers(linenumbers);
		Method method = inst.getClass().getMethod("setSourceLineNumbers", Set.class );
		method.setAccessible(true);
		method.invoke(inst, lineNumbers);

		return inst;
	}

	List<String> listIgnore = Arrays.asList(
		"}",
		"fi", "else", "elif", "esac", "do", "done"
	);

	Set<Integer> parseLineNumbers(String shell) {
		Set<Integer> ret = new TreeSet<Integer>();
		try {
			int i = 0;
			BufferedReader rd = new BufferedReader(new FileReader(shell));
			for ( ;; ) {
				String line = rd.readLine();
				if (line == null) {
					break;
				}
				i++;
				line = line.trim();
				if (line.length() == 0 || line.startsWith("#")) {
					continue;
				}
				if (line.endsWith("\\")) {
					continue;
				}
				if (line.startsWith("function ") && line.endsWith("{")) {
					continue;
				}
				if (listIgnore.contains(line)) {
					continue;
				}
				ret.add(i);
			}
			rd.close();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		return ret;
	}

	public String getShell() {
		return shell;
	}

}
